package tests

import (
	"testing"

	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"

	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/testutils/apitester"
)

func TestUserManagement(t *testing.T) {
	tester := apitester.NewAPITester(t, nil)
	tester.Debug()

	defer tester.Close()

	defer tester.Logout()
	tester.Login("admin", "admin")

	_ = t.Run("Create", func(t *testing.T) {
		defer tester.SetT(t)()
		tester.CreateUser("janedoe", "janedoe")
	}) &&
		t.Run(
			"Get",
			func(t *testing.T) {
				defer tester.SetT(t)()
				var info models.Userinfo
				tester.GetUser("janedoe", &info)
				assert.False(t, info.IsAdmin)
			}) &&
		t.Run(
			"List",
			func(t *testing.T) {
				defer tester.SetT(t)()
				var infoList []*models.Userinfo
				tester.ListUsers(&infoList)

				require.Equal(t, 2, len(infoList))
				assert.Equal(t, "admin", infoList[0].Username)
				assert.True(t, infoList[0].IsAdmin)
				assert.Equal(t, "janedoe", infoList[1].Username)
				assert.False(t, infoList[1].IsAdmin)
				admin := models.Userinfo{
					Username: "admin",
					IsAdmin:  true,
					Activity: &models.Activity{},
				}
				assert.Equal(t, admin, *infoList[0])
				janedoe := models.Userinfo{
					Username:  "janedoe",
					Email:     "janedoe@mydomain.fr",
					IsAdmin:   false,
					CreatedAt: infoList[1].CreatedAt,
					Activity: &models.Activity{
						RenderStats: &models.RenderStats{
							Total:        int64(0),
							CurrentMonth: int64(0),
							LastMonth:    int64(0),
						},
					},
				}
				assert.Equal(t, janedoe, *infoList[1])
			})
}
