package utils

import (
	"time"

	"github.com/prometheus/client_golang/prometheus"
)

// VecTimer is a helper type to time functions. Use NewVecTimer to create new
// instances.
type VecTimer struct {
	begin time.Time
	v     prometheus.ObserverVec
}

// NewVecTimer creates a new VecTimer.
func NewVecTimer(v prometheus.ObserverVec) *VecTimer {
	return &VecTimer{
		begin: time.Now(),
		v:     v,
	}
}

func (t VecTimer) GetBegin() time.Time {
	return t.begin
}

// ObserveDuration records the duration passed since the Timer was created with
// NewTimer. It calls the Observe method of the Observer provided during
// construction with the duration in seconds as an argument. The observed
// duration is also returned. ObserveDuration is usually called with a defer
// statement.
//
// Note that this method is only guaranteed to never observe negative durations
// if used with Go1.9+.
func (t *VecTimer) ObserveDurationWithLabelValues(values ...string) time.Duration {
	d := time.Since(t.begin)
	if t.v != nil {
		t.v.WithLabelValues(values...).Observe(d.Seconds())
	}

	return d
}
